==> yin_azlistview-0.0.8.tar.gz: lib/yin/AzListView.dart <==
import 'package:flutter/material.dart';
import 'package:lpinyin/lpinyin.dart';
import 'package:flustars/flustars.dart';
import 'package:yin_azlistview/yin/AzBean.dart';

class AzListView extends StatefulWidget {
  final List<AzBean> topList;
  final List<AzBean> cityList;

  final Function itemView;
  final int itemHeight;

  final Function headView;
  final int headHeight;

  final Function tagView;
  final int tagHeight;

  final bool displayRight; //右边列表是否显示
  final bool displayRightOther; //右边列表-是否显示非A-Z数据

  AzListView(
    this.cityList, {
    this.topList,
    this.itemView,
    this.itemHeight = 50,
    this.headView,
    this.headHeight = 30,
    this.tagView,
    this.tagHeight = 20,
    this.displayRight = true,
    this.displayRightOther = true,
  });

  @override
  AzListViewState createState() => new AzListViewState();
}

class AzListViewState extends State<AzListView> {
  List<AzBean> displayList = List(); //列表显示数据(包括head)
  List<AzBean> rightTags = new List(); //右边列表数据

  int diffHeight = 0; //悬浮head距离顶部高度
  String headValue; //悬浮head的值
  String middletTag; //右边触摸时，中间显示值

  List<int> listHeights = new List(); //列表数据对应的高度
  List<int> listRights = new List(); // 右边列表对应的高度

  int toUpHeight = 0; //
  int screenHeight = ScreenUtil.getInstance().screenHeight.toInt();
  bool displayMiddle = false;

  ScrollController controller;

  @override
  void initState() {
    super.initState();
    init();
  }

  @override
  void didUpdateWidget(AzListView oldWidget) {
    super.didUpdateWidget(oldWidget);
    loadData();
  }

  void init() {
    controller = ScrollController();
    controller.addListener(() {
      int offset = controller.offset.toInt();
      int curPosition = 0;
      for (var i = 1; i < listHeights.length - 1; ++i) {
        int top = listHeights[i];
        int bottom = listHeights[i] + (displayList[i].isHeader ? widget.headHeight : widget.itemHeight);
        if (offset >= top && offset < bottom) {
          curPosition = i;
          break;
        }
      }
      if (displayList[curPosition].isHeader) {
        setState(() {
          diffHeight = 0;
          headValue = displayList[curPosition].headTag;
        });
      } else if (displayList[curPosition + 1].isHeader) {
        int number = listHeights[curPosition] - offset;
        if (number < -widget.headHeight) {
          setState(() {
            diffHeight = number + widget.headHeight;
            headValue = displayList[curPosition].headTag;
          });
        }
      } else {
        setState(() {
          diffHeight = 0;
          headValue = displayList[curPosition].headTag;
        });
      }
    });
  }

  void loadData() {
    sortList(widget.cityList);

    listHeights.clear();
    rightTags.clear();
    displayList.clear();

    int tempListHeight = 0;
    int tempRightHeight = 0;

    initData(widget.topList);
    initData(widget.cityList);

    for (int i = 0; i < displayList.length; i++) {
      if (i == 0) {
        listHeights.add(tempListHeight);
      } else if (displayList[i - 1].isHeader) {
        listHeights.add(tempListHeight += widget.headHeight);
      } else {
        listHeights.add(tempListHeight += widget.itemHeight);
      }
    }

    for (var i = 0; i < rightTags.length; ++i) {
      if (i == 0) {
        listRights.add(tempRightHeight);
      } else {
        listRights.add(tempRightHeight += widget.tagHeight);
      }
    }
    toUpHeight = (screenHeight - listRights.length * widget.tagHeight) ~/ 2;
    setState(() {});
  }

  /// item-头部
  Widget itemHeaderView(String value) {
    return Container(
      height: widget.headHeight.toDouble(),
      width: double.infinity,
      color: Colors.grey[200],
      padding: EdgeInsets.only(left: 16, right: 16),
      alignment: Alignment.centerLeft,
      child: Text(value ?? ""),
    );
  }

  /// item-内容
  Widget itemContentView(AzBean bean) {
    return Container(
      height: widget.itemHeight.toDouble(),
      child: new Text(bean.name),
      color: Colors.white,
      alignment: Alignment.centerLeft,
      padding: EdgeInsets.only(left: 16, right: 16),
    );
  }

  ///列表-item
  Widget itemView(int position) {
    AzBean bean = displayList[position];
    if (bean.isHeader) {
      if (widget.headView == null) {
        return itemHeaderView(bean.headTag);
      } else {
        return widget.headView(bean);
      }
    } else {
      if (widget.itemView == null) {
        return itemContentView(bean);
      } else {
        return widget.itemView(bean);
      }
    }
  }

  ///列表视图
  Widget listView() {
    return ListView.builder(
      itemBuilder: (context, itemIndex) => itemView(itemIndex),
      itemCount: displayList.length,
      controller: controller,
    );
  }

  /// 列表悬停视图
  Widget tagView() {
    Widget view;
    if (widget.tagView == null) {
      view = itemHeaderView(headValue);
    } else {
      view = widget.tagView(headValue);
    }
    return Positioned(
      top: diffHeight.toDouble(),
      left: 0,
      right: 0,
      child: view,
    );
  }

  ///右边视图
  Widget rightTagView() {
    List<Widget> views = new List();
    rightTags.forEach((it) {
      views.add(GestureDetector(
        onTapUp: (up) {
          setState(() {
            displayMiddle = false;
          });
        },
        onTapDown: (down) {
          setState(() {
            displayMiddle = true;
            headValue = it.headTag;
            middletTag = it.rightTag;
          });
          scrollTo(it);
        },
        onVerticalDragUpdate: (DragUpdateDetails details) {
          rightItemPosition(details.globalPosition.dy.toInt());
        },
        onVerticalDragEnd: (DragEndDetails details) {
          setState(() {
            displayMiddle = false;
          });
        },
        child: Container(
          width: 20,
          height: 20,
          color: Colors.grey[200],
          child: Text(
            it.rightTag,
            textAlign: TextAlign.center,
          ),
        ),
      ));
    });

    return Align(
      alignment: Alignment.centerRight,
      child: Column(
        children: views,
        mainAxisAlignment: MainAxisAlignment.center,
      ),
    );
  }

  ///中间视图
  Widget middleView() {
    return Align(
      alignment: Alignment.center,
      child: Container(
        height: 60,
        width: 60,
        alignment: Alignment.center,
        color: Colors.green,
        child: Text(
          middletTag,
          style: TextStyle(fontSize: 32, color: Colors.white),
        ),
      ),
    );
  }

  ///内容视图
  Widget contentView() {
    List<Widget> views = new List();
    //列表视图
    views.add(listView());

    //列表悬停视图
    views.add(tagView());

    //右边视图
    if (widget.displayRight) {
      views.add(rightTagView());
    }

    //中间视图
    if (displayMiddle) {
      views.add(middleView());
    }

    return Stack(children: views);
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      child: contentView(),
    );
  }

  ///数据排序
  void sortList(List<AzBean> list) {
    if (list == null || list.isEmpty) return;
    list.forEach((it) {
      String pinyin = PinyinHelper.getPinyinE(it.name);
      String tag = pinyin.substring(0, 1).toUpperCase();
      if (RegExp("[A-Z]").hasMatch(tag)) {
        it.headTag = tag;
        it.rightTag = tag;
      } else {
        it.headTag = "#";
        it.rightTag = "#";
      }
    });

    list.sort((left, right) {
      if (left.headTag == "@" || right.headTag == "#") {
        return -1;
      } else if (left.headTag == "#" || right.headTag == "@") {
        return 1;
      } else {
        return left.headTag.compareTo(right.headTag);
      }
    });
  }

  ///包装数据
  void initData(List<AzBean> list) {
    if (list == null) return;
    String curTag = "";
    for (var i = 0; i < list.length; ++i) {
      var bean = list[i];
      if (bean.headTag != curTag) {
        curTag = bean.headTag;
        displayList.add(AzBean(headTag: curTag, rightTag: bean.rightTag, isHeader: true));
      }
      displayList.add(bean);
      if (headValue == null) {
        headValue = bean.headTag;
      }

      if (widget.displayRightOther || RegExp("[A-Z]").hasMatch(bean.rightTag)) {
        if (rightTags.firstWhere((it) => it.rightTag == bean.rightTag, orElse: () => null) == null) {
          rightTags.add(bean);
        }
      }
    }
  }

  ///判定右侧位置
  void rightItemPosition(int scrollDy) {
    int index = (scrollDy - toUpHeight - 40) ~/ widget.tagHeight;
    if (index >= 0 && index < rightTags.length) {
      setState(() {
        middletTag = rightTags[index].rightTag;
        headValue = rightTags[index].headTag;
        displayMiddle = true;
      });

      scrollTo(rightTags[index]);
    } else {
      setState(() {
        displayMiddle = false;
      });
    }
  }

  ///滚动到指定位置
  void scrollTo(AzBean tag) {
    int index = 0;
    for (int i = 0; i < displayList.length; i++) {
      if (displayList[i].headTag == tag.headTag) {
        index = i;
        break;
      }
    }
    controller.jumpTo(listHeights[index].toDouble());
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}

==> yin_azlistview-0.1.1.tar.gz: lib/yin/AzListView.dart <==
import 'package:flutter/material.dart';
import 'package:lpinyin/lpinyin.dart';
import 'package:flustars/flustars.dart';
import 'package:yin_azlistview/yin/AzBean.dart';

class AzListView extends StatefulWidget {
  final List<AzBean> topList;
  final List<AzBean> cityList;

  final Function itemView;
  final int itemHeight;

  final Function headView;
  final int headHeight;

  final Function tagView;
  final int tagHeight;

  final bool displayRight; //右边列表是否显示
  final bool displayRightOther; //右边列表-是否显示非A-Z数据

  AzListView(
    this.cityList, {
    this.topList,
    this.itemView,
    this.itemHeight = 50,
    this.headView,
    this.headHeight = 30,
    this.tagView,
    this.tagHeight = 20,
    this.displayRight = true,
    this.displayRightOther = true,
  });

  @override
  AzListViewState createState() => new AzListViewState();
}

class AzListViewState extends State<AzListView> {
  List<AzBean> displayList = List(); //列表显示数据(包括head)
  List<AzBean> rightTags = new List(); //右边列表数据

  int diffHeight = 0; //悬浮head距离顶部高度
  String headValue; //悬浮head的值
  String middletTag; //右边触摸时，中间显示值

  List<int> listHeights = new List(); //列表数据对应的高度
  List<int> listRights = new List(); // 右边列表对应的高度

  int screenHeight = ScreenUtil.getInstance().screenHeight.toInt();
  bool displayMiddle = false;

  ScrollController controller;

  @override
  void initState() {
    super.initState();
    init();
  }

  @override
  void didUpdateWidget(AzListView oldWidget) {
    super.didUpdateWidget(oldWidget);
    loadData();
  }

  void init() {
    controller = ScrollController();
    controller.addListener(() {
      int offset = controller.offset.toInt();
      int curPosition = 0;
      for (var i = 1; i < listHeights.length - 1; ++i) {
        int top = listHeights[i];
        int bottom = listHeights[i] + (displayList[i].isHeader ? widget.headHeight : widget.itemHeight);
        if (offset >= top && offset < bottom) {
          curPosition = i;
          break;
        }
      }
      if (displayList[curPosition].isHeader) {
        setState(() {
          diffHeight = 0;
          headValue = displayList[curPosition].headTag;
        });
      } else if (displayList[curPosition + 1].isHeader) {
        int number = listHeights[curPosition] - offset;
        if (number < -widget.headHeight) {
          setState(() {
            diffHeight = number + widget.headHeight;
            headValue = displayList[curPosition].headTag;
          });
        }
      } else {
        setState(() {
          diffHeight = 0;
          headValue = displayList[curPosition].headTag;
        });
      }
    });
  }

  void loadData() {
    sortList(widget.cityList);

    listHeights.clear();
    rightTags.clear();
    listRights.clear();
    displayList.clear();

    int tempListHeight = 0;
    int tempRightHeight = 0;

    initData(widget.topList);
    initData(widget.cityList);

    for (int i = 0; i < displayList.length; i++) {
      if (i == 0) {
        listHeights.add(tempListHeight);
      } else if (displayList[i - 1].isHeader) {
        listHeights.add(tempListHeight += widget.headHeight);
      } else {
        listHeights.add(tempListHeight += widget.itemHeight);
      }
    }

    for (var i = 0; i < rightTags.length; ++i) {
      if (i == 0) {
        listRights.add(tempRightHeight);
      } else {
        listRights.add(tempRightHeight += widget.tagHeight);
      }
    }
    setState(() {});
  }

  /// item-头部
  Widget itemHeaderView(String value) {
    return Container(
      height: widget.headHeight.toDouble(),
      width: double.infinity,
      color: Colors.grey[200],
      padding: EdgeInsets.only(left: 16, right: 16),
      alignment: Alignment.centerLeft,
      child: Text(value ?? ""),
    );
  }

  /// item-内容
  Widget itemContentView(AzBean bean) {
    return Container(
      height: widget.itemHeight.toDouble(),
      child: new Text(bean.name),
      color: Colors.white,
      alignment: Alignment.centerLeft,
      padding: EdgeInsets.only(left: 16, right: 16),
    );
  }

  ///列表-item
  Widget itemView(int position) {
    AzBean bean = displayList[position];
    if (bean.isHeader) {
      if (widget.headView == null) {
        return itemHeaderView(bean.headTag);
      } else {
        return widget.headView(bean);
      }
    } else {
      if (widget.itemView == null) {
        return itemContentView(bean);
      } else {
        return widget.itemView(bean);
      }
    }
  }

  ///列表视图
  Widget listView() {
    return ListView.builder(
      itemBuilder: (context, itemIndex) => itemView(itemIndex),
      itemCount: displayList.length,
      controller: controller,
    );
  }

  /// 列表悬停视图
  Widget tagView() {
    Widget view;
    if (widget.tagView == null) {
      view = itemHeaderView(headValue);
    } else {
      view = widget.tagView(headValue);
    }
    return Positioned(
      top: diffHeight.toDouble(),
      left: 0,
      right: 0,
      child: view,
    );
  }

  ///右边视图
  Widget rightTagView() {
    List<Widget> views = new List();
    rightTags.forEach((it) {
      views.add(Expanded(
        child: GestureDetector(
          onTapUp: (up) {
            setState(() {
              displayMiddle = false;
            });
          },
          onTapDown: (down) {
            setState(() {
              displayMiddle = true;
              headValue = it.headTag;
              middletTag = it.rightTag;
            });
            scrollTo(it);
          },
          onVerticalDragUpdate: (DragUpdateDetails details) {
            rightItemPosition(details.globalPosition.dy.toInt());
          },
          onVerticalDragEnd: (DragEndDetails details) {
            setState(() {
              displayMiddle = false;
            });
          },
          child: Container(
            width: 20,
            color: Colors.grey[200],
            alignment: Alignment.center,
            child: Text(
              it.rightTag,
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
              textAlign: TextAlign.center,
            ),
          ),
        ),
      ));
    });

    return Container(
      alignment: Alignment.centerRight,
      margin: EdgeInsets.only(top: widget.headHeight.toDouble()),
      child: Column(
        children: views,
        mainAxisAlignment: MainAxisAlignment.spaceAround,
      ),
    );
  }

  ///中间视图
  Widget middleView() {
    return Align(
      alignment: Alignment.center,
      child: Container(
        height: 60,
        width: 60,
        alignment: Alignment.center,
        color: Colors.green,
        child: Text(
          middletTag,
          maxLines: 1,
          overflow: TextOverflow.ellipsis,
          style: TextStyle(fontSize: 32, color: Colors.white),
        ),
      ),
    );
  }

  ///内容视图
  Widget contentView() {
    List<Widget> views = new List();
    //列表视图
    views.add(listView());

    //列表悬停视图
    views.add(tagView());

    //右边视图
    if (widget.displayRight) {
      views.add(rightTagView());
    }

    //中间视图
    if (displayMiddle) {
      views.add(middleView());
    }

    return Stack(children: views);
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      child: contentView(),
    );
  }

  ///数据排序
  void sortList(List<AzBean> list) {
    if (list == null || list.isEmpty) return;
    list.forEach((it) {
      String pinyin = PinyinHelper.getPinyinE(it.name);
      String tag = pinyin.substring(0, 1).toUpperCase();
      if (RegExp("[A-Z]").hasMatch(tag)) {
        it.headTag = tag;
        it.rightTag = tag;
      } else {
        it.headTag = "#";
        it.rightTag = "#";
      }
    });

    list.sort((left, right) {
      if (left.headTag == "@" || right.headTag == "#") {
        return -1;
      } else if (left.headTag == "#" || right.headTag == "@") {
        return 1;
      } else {
        return left.headTag.compareTo(right.headTag);
      }
    });
  }

  ///包装数据
  void initData(List<AzBean> list) {
    if (list == null) return;
    String curTag = "";
    for (var i = 0; i < list.length; ++i) {
      var bean = list[i];
      if (bean.headTag != curTag) {
        curTag = bean.headTag;
        displayList.add(AzBean(headTag: curTag, rightTag: bean.rightTag, isHeader: true));
      }
      displayList.add(bean);
      if (headValue == null) {
        headValue = bean.headTag;
      }

      if (widget.displayRightOther || RegExp("[A-Z]").hasMatch(bean.rightTag) || bean.rightTag == "#") {
        if (rightTags.firstWhere((it) => it.rightTag == bean.rightTag, orElse: () => null) == null) {
          rightTags.add(bean);
        }
      }
    }
  }

  ///判定右侧位置
  void rightItemPosition(int scrollDy) {
    int index = (scrollDy - 40) ~/ widget.tagHeight;
    if (index >= 0 && index < rightTags.length) {
      setState(() {
        middletTag = rightTags[index].rightTag;
        headValue = rightTags[index].headTag;
        displayMiddle = true;
      });

      scrollTo(rightTags[index]);
    } else {
      setState(() {
        displayMiddle = false;
      });
    }
  }

  ///滚动到指定位置
  void scrollTo(AzBean tag) {
    int index = 0;
    for (int i = 0; i < displayList.length; i++) {
      if (displayList[i].headTag == tag.headTag) {
        index = i;
        break;
      }
    }
    controller.jumpTo(listHeights[index].toDouble());
  }

  @override
  void dispose() {
    controller.dispose();
    super.dispose();
  }
}
